@@ -6,14 +6,17 @@ File based database interface
 """
 
 import datetime
-from collections.abc import Iterable
-import numpy as np
 import os
+from abc import ABCMeta
+from collections.abc import Iterable
+from typing import List, Optional, Union
+
+import numpy as np
 import pandas
+from h5io_browser.base import _read_hdf, _write_hdf
 from pyfileindex import PyFileIndex
-from pyiron_base.interfaces.singleton import Singleton
-from pyiron_base.database.generic import IsDatabase
-from pyiron_base.storage.helper_functions import read_hdf5, write_hdf5
+
+from pyiron_base.database.interface import IsDatabase
 
 __author__ = "Jan Janssen"
 __copyright__ = (
@@ -47,15 +50,58 @@ table_columns = {
 }
 
 
-class FileTable(IsDatabase, metaclass=Singleton):
-    def __init__(self, project):
+class FileTableSingleton(ABCMeta):
+    """
+    Indexing the file system for each `FileTable` can be expensive, so we use a
+    singleton system that does this once for each path instead.
+    """
+
+    _instances = {}
+
+    def __call__(cls, index_from: str):
+        path = os.path.abspath(os.path.expanduser(index_from))
+        if path not in cls._instances:
+            cls._instances[path] = super().__call__(
+                index_from=path,
+                fileindex=cls._get_fileindex_if_theres_a_common_path(path),
+            )
+        return cls._instances[path]
+
+    def _get_fileindex_if_theres_a_common_path(
+        cls, path: str
+    ) -> Union[PyFileIndex, None]:
+        common_path = _get_most_common_path(
+            path=path, reference_paths=cls._instances.keys()
+        )
+        if common_path is not None:
+            return cls._instances[common_path]._fileindex.open(path)
+        else:
+            return None
+
+
+class FileTable(IsDatabase, metaclass=FileTableSingleton):
+    """
+    File table should behave to the user like a database, but it infers project
+    hierarchy directly from the file system hierarchy.
+
+    Because indexing the file system can be expensive, and projects sometimes get
+    re-initialized, it is important to keep the (re)instantiation cost for this class
+    as minimal as possible.
+
+    Args:
+         index_from (str): The file path to start indexing at, i.e. the project path.
+         fileindex (PyFileIndex): In case the file path in index_from is already indexed,
+                                  then the index can be provided as additional input parameter.
+    """
+
+    def __init__(self, index_from: str, fileindex: PyFileIndex = None):
         self._fileindex = None
         self._job_table = None
-        self._project = os.path.abspath(project)
+        self._path = os.path.abspath(index_from)
         self._columns = list(table_columns.keys())
-        self.force_reset()
+        self.force_reset(fileindex=fileindex)
 
-    def add_item_dict(self, par_dict):
+    def add_item_dict(self, par_dict: dict) -> int:
         """
         Create a new database item
 
@@ -99,7 +145,7 @@ class FileTable(IsDatabase, metaclass=Singleton):
         ).reset_index(drop=True)
         return int(par_dict_merged["id"])
 
-    def delete_item(self, item_id):
+    def delete_item(self, item_id: int) -> None:
         """
         Delete Item from database
 
@@ -114,13 +160,19 @@ class FileTable(IsDatabase, metaclass=Singleton):
         else:
             raise ValueError
 
-    def force_reset(self):
+    def force_reset(self, fileindex: Optional[PyFileIndex] = None) -> None:
         """
         Reset cache of the FileTable object
+
+        Args:
+            fileindex (PyFileIndex): File index for the current directory
         """
-        self._fileindex = PyFileIndex(
-            path=self._project, filter_function=filter_function
-        )
+        if fileindex is not None:
+            self._fileindex = fileindex
+        else:
+            self._fileindex = PyFileIndex(
+                path=self._path, filter_function=filter_function
+            )
         df = pandas.DataFrame(self.init_table(fileindex=self._fileindex.dataframe))
         if len(df) != 0:
             df.id = df.id.astype(int)
@@ -128,7 +180,12 @@ class FileTable(IsDatabase, metaclass=Singleton):
         else:
             self._job_table = pandas.DataFrame({k: [] for k in self._columns})
 
-    def get_child_ids(self, job_specifier, project=None, status=None):
+    def get_child_ids(
+        self,
+        job_specifier: Union[str, int],
+        project: Optional[str] = None,
+        status: Optional[str] = None,
+    ) -> List[int]:
         """
         Get the childs for a specific job
 
@@ -141,7 +198,7 @@ class FileTable(IsDatabase, metaclass=Singleton):
             list: list of child IDs
         """
         if project is None:
-            project = self._project
+            project = self._path
         self.update()
         id_master = self.get_job_id(project=project, job_specifier=job_specifier)
         if id_master is None:
@@ -160,7 +217,7 @@ class FileTable(IsDatabase, metaclass=Singleton):
                 ].id.values
             return sorted(id_lst)
 
-    def get_item_by_id(self, item_id):
+    def get_item_by_id(self, item_id: int) -> dict:
         """
         Get item from database by searching for a specific item Id.
 
@@ -192,7 +249,9 @@ class FileTable(IsDatabase, metaclass=Singleton):
             for k, v in self._job_table[self._job_table.id == item_id].to_dict().items()
         }
 
-    def get_items_dict(self, item_dict, return_all_columns=True):
+    def get_items_dict(
+        self, item_dict: dict, return_all_columns: bool = True
+    ) -> List[dict]:
         """
         Get list of jobs which fulfills the query in the dictionary
 
@@ -267,11 +326,19 @@ class FileTable(IsDatabase, metaclass=Singleton):
         else:
             return [{"id": i} for i in df_dict["id"].values()]
 
-    def get_jobs(self, project=None, recursive=True, columns=None):
+    def _get_jobs(
+        self,
+        user: str,
+        sql_query: str,
+        project: Optional[str] = None,
+        recursive: bool = True,
+        columns: Optional[List[str]] = None,
+    ) -> dict:
         """
         Get jobs as dictionary from filetable
 
         Args:
+            user/sql_query: ignored for compat with IsDatabase
             project (str/ None): path to the project
             recursive (boolean): recursively iterate over all sub projects
             columns (list/ None): list of columns to return
@@ -280,7 +347,7 @@ class FileTable(IsDatabase, metaclass=Singleton):
             dict: job entries as dictionary
         """
         if project is None:
-            project = self._project
+            project = self._path
         if columns is None:
             columns = ["id", "project"]
         df = self.job_table(
@@ -303,20 +370,9 @@ class FileTable(IsDatabase, metaclass=Singleton):
             ].tolist()  # ToDo: Check difference of tolist and to_list
         return dictionary
 
-    def get_job_ids(self, project=None, recursive=True):
-        """
-        Get job IDs from filetable
-
-        Args:
-            project (str/ None): path to the project
-            recursive (boolean): recursively iterate over all sub projects
-
-        Returns:
-            list/ None: list of job IDs
-        """
-        return self.get_jobs(project=project, recursive=recursive, columns=["id"])["id"]
-
-    def get_job_id(self, job_specifier, project=None):
+    def get_job_id(
+        self, job_specifier: Union[str, int], project: Optional[str] = None
+    ) -> int:
         """
         Get job ID from filetable
 
@@ -328,7 +384,7 @@ class FileTable(IsDatabase, metaclass=Singleton):
             int/ None: job ID
         """
         if project is None:
-            project = self._project
+            project = self._path
         if isinstance(job_specifier, (int, np.integer)):
             return job_specifier  # is id
         if len(self._job_table) == 0:
@@ -352,7 +408,7 @@ class FileTable(IsDatabase, metaclass=Singleton):
                 "job name '{0}' in this project is not unique".format(job_specifier)
             )
 
-    def get_job_status(self, job_id):
+    def get_job_status(self, job_id: int) -> str:
         """
         Get status of a given job selected by its job ID
 
@@ -364,7 +420,7 @@ class FileTable(IsDatabase, metaclass=Singleton):
         """
         return self._job_table[self._job_table.id == job_id].status.values[0]
 
-    def get_job_working_directory(self, job_id):
+    def get_job_working_directory(self, job_id: int) -> Union[str, None]:
         """
         Get the working directory of a particular job
 
@@ -388,7 +444,9 @@ class FileTable(IsDatabase, metaclass=Singleton):
         except KeyError:
             return None
 
-    def init_table(self, fileindex, working_dir_lst=None):
+    def init_table(
+        self, fileindex: PyFileIndex, working_dir_lst: Optional[List[str]] = None
+    ) -> List[dict]:
         """
         Initialize the filetable class
 
@@ -423,7 +481,7 @@ class FileTable(IsDatabase, metaclass=Singleton):
                 job_lst.append(job_dict)
         return job_lst
 
-    def _item_update(self, par_dict, item_id):
+    def _item_update(self, par_dict: dict, item_id: int) -> None:
         """
         Modify Item in database
 
@@ -439,7 +497,7 @@ class FileTable(IsDatabase, metaclass=Singleton):
         for k, v in par_dict.items():
             self._job_table.loc[self._job_table.id == int(item_id), k] = v
 
-    def set_job_status(self, job_id, status):
+    def set_job_status(self, job_id: int, status: str) -> None:
         """
         Set job status
 
@@ -450,26 +508,33 @@ class FileTable(IsDatabase, metaclass=Singleton):
         super().set_job_status(job_id=job_id, status=status)
         self._update_hdf5_status(job_id=job_id, status=status)
 
-    def _update_hdf5_status(self, job_id, status):
+    def _update_hdf5_status(self, job_id: int, status: str) -> None:
+        """
+        Update the status of a job in its HDF5 file.
+
+        Args:
+            job_id (int): The ID of the job.
+            status (str): The new status of the job.
+        """
         if isinstance(job_id, Iterable):
             for j_id in job_id:
                 db_entry = self.get_item_by_id(item_id=j_id)
-                write_hdf5(
-                    db_entry["project"] + db_entry["subjob"] + ".h5",
-                    status,
-                    title=db_entry["subjob"][1:] + "/status",
+                _write_hdf(
+                    hdf_filehandle=db_entry["project"] + db_entry["subjob"] + ".h5",
+                    data=status,
+                    h5_path=db_entry["subjob"][1:] + "/status",
                     overwrite="update",
                 )
         else:
             db_entry = self.get_item_by_id(item_id=job_id)
-            write_hdf5(
-                db_entry["project"] + db_entry["subjob"] + ".h5",
-                status,
-                title=db_entry["subjob"][1:] + "/status",
+            _write_hdf(
+                hdf_filehandle=db_entry["project"] + db_entry["subjob"] + ".h5",
+                data=status,
+                h5_path=db_entry["subjob"][1:] + "/status",
                 overwrite="update",
             )
 
-    def update(self):
+    def update(self) -> None:
         """
         Update the filetable cache
         """
@@ -487,9 +552,15 @@ class FileTable(IsDatabase, metaclass=Singleton):
                     )
                 ]
             )
+            sanitized_paths = self._fileindex.dataframe.path.str.replace("\\", "/")
+            # The files_list is generated using project path values
+            # In pyiron, these are all forced to be posix-like with /
+            # But _fileindex is of type PyFileIndex, which does _not_ modify paths
+            # so to get the two compatible for an isin check, we need to sanitize the
+            # _fileindex.dataframe.path results
             df_new = self._fileindex.dataframe[
                 ~self._fileindex.dataframe.is_directory
-                & ~self._fileindex.dataframe.path.isin(files_lst)
+                & ~sanitized_paths.isin(files_lst)
             ]
         else:
             files_lst, working_dir_lst = [], []
@@ -508,7 +579,17 @@ class FileTable(IsDatabase, metaclass=Singleton):
                     self._job_table = df
 
     @staticmethod
-    def get_extract(path, mtime):
+    def get_extract(path: str, mtime: datetime.datetime) -> dict:
+        """
+        Extract job information from a given file path and modification time.
+
+        Args:
+            path (str): The file path.
+            mtime (datetime.datetime): The modification time.
+
+        Returns:
+            dict: A dictionary containing the extracted job information.
+        """
         basename = os.path.basename(path)
         job = os.path.splitext(basename)[0]
         time = datetime.datetime.fromtimestamp(mtime)
@@ -518,7 +599,11 @@ class FileTable(IsDatabase, metaclass=Singleton):
                 "status": get_job_status_from_file(hdf5_file=path, job_name=job),
                 "job": job,
                 "subjob": "/" + job,
-                "project": os.path.dirname(path) + "/",
+                "project": os.path.dirname(path).replace("\\", "/") + "/",
+                # pyiron Project paths are forced to be posix-like with / instead of \
+                # in order for the contains and endswith tests down in _get_job_table
+                # to work on windows, we need to make sure that the file table obeys
+                # this conversion
                 "timestart": time,
                 "timestop": time,
                 "totalcputime": 0.0,
@@ -532,7 +617,16 @@ class FileTable(IsDatabase, metaclass=Singleton):
         del return_dict["masterid"]
         return return_dict
 
-    def _get_job_status_from_hdf5(self, job_id):
+    def _get_job_status_from_hdf5(self, job_id: int) -> str:
+        """
+        Get the status of a job from its HDF5 file.
+
+        Args:
+            job_id (int): The ID of the job.
+
+        Returns:
+            str: The status of the job.
+        """
         db_entry = self.get_item_by_id(job_id)
         job_name = db_entry["subjob"][1:]
         return get_job_status_from_file(
@@ -542,27 +636,43 @@ class FileTable(IsDatabase, metaclass=Singleton):
 
     def _get_job_table(
         self,
-        sql_query,
-        user,
-        project_path=None,
-        recursive=True,
-        columns=None,
-        element_lst=None,
-    ):
+        sql_query: str,
+        user: str,
+        project_path: Optional[str] = None,
+        recursive: bool = True,
+        columns: Optional[List[str]] = None,
+        element_lst: Optional[List[str]] = None,
+    ) -> pandas.DataFrame:
+        """
+        Get the job table based on the specified parameters.
+
+        Args:
+            sql_query (str): SQL query string.
+            user (str): User name.
+            project_path (str, optional): Project path. Defaults to None.
+            recursive (bool, optional): Recursive flag. Defaults to True.
+            columns (List[str], optional): List of columns to return. Defaults to None.
+            element_lst (List[str], optional): List of elements. Defaults to None.
+
+        Returns:
+            pandas.DataFrame: The job table.
+        """
         self.update()
         if project_path is None:
-            project_path = self._project
+            project_path = self._path
         if len(self._job_table) != 0:
             if recursive:
                 return self._job_table[
                     self._job_table.project.str.contains(project_path)
                 ]
             else:
-                return self._job_table[self._job_table.project == project_path]
+                return self._job_table[
+                    self._job_table.project.str.endswith(project_path)
+                ]
         else:
             return self._job_table
 
-    def _get_table_headings(self, table_name=None):
+    def _get_table_headings(self, table_name: Optional[str] = None) -> List[str]:
         """
         Get column names
 
@@ -590,24 +700,91 @@ class FileTable(IsDatabase, metaclass=Singleton):
         """
         return self._job_table.columns.values
 
-    def _get_view_mode(self):
+    def _get_view_mode(self) -> bool:
+        """
+        Get the view mode of the file table.
+
+        Returns:
+            bool: The view mode of the file table - Always False.
+        """
         return False
 
 
-def filter_function(file_name):
+def filter_function(file_name: str) -> bool:
+    """
+    Filter function to check if a file name contains ".h5".
+
+    Args:
+        file_name (str): The name of the file.
+
+    Returns:
+        bool: True if the file name contains ".h5", False otherwise.
+    """
     return ".h5" in file_name
 
 
-def get_hamilton_from_file(hdf5_file, job_name):
-    return read_hdf5(hdf5_file, job_name + "/TYPE").split(".")[-1].split("'")[0]
+def get_hamilton_from_file(hdf5_file: str, job_name: str) -> str:
+    """
+    Get the Hamilton type from an HDF5 file.
+
+    Args:
+        hdf5_file (str): The path to the HDF5 file.
+        job_name (str): The name of the job.
+
+    Returns:
+        str: The Hamilton type.
+    """
+    return (
+        _read_hdf(hdf_filehandle=hdf5_file, h5_path=job_name + "/TYPE")
+        .split(".")[-1]
+        .split("'")[0]
+    )
 
 
-def get_hamilton_version_from_file(hdf5_file, job_name):
-    return read_hdf5(hdf5_file, job_name + "/VERSION")
+def get_hamilton_version_from_file(hdf5_file: str, job_name: str) -> str:
+    """
+    Get the Hamilton version from an HDF5 file.
+
+    Args:
+        hdf5_file (str): The path to the HDF5 file.
+        job_name (str): The name of the job.
+
+    Returns:
+        str: The Hamilton version.
+    """
+    return _read_hdf(hdf_filehandle=hdf5_file, h5_path=job_name + "/VERSION")
 
 
-def get_job_status_from_file(hdf5_file, job_name):
+def get_job_status_from_file(hdf5_file: str, job_name: str) -> Union[str, None]:
+    """
+    Get the status of a job from an HDF5 file.
+
+    Args:
+        hdf5_file (str): The path to the HDF5 file.
+        job_name (str): The name of the job.
+
+    Returns:
+        Union[str, None]: The status of the job, or None if the file does not exist.
+    """
     if os.path.exists(hdf5_file):
-        return read_hdf5(hdf5_file, job_name + "/status")
+        return _read_hdf(hdf_filehandle=hdf5_file, h5_path=job_name + "/status")
+    else:
+        return None
+
+
+def _get_most_common_path(path: str, reference_paths: List[str]) -> Union[str, None]:
+    """
+    Get the most common path between the given path and a list of reference paths.
+
+    Args:
+        path (str): The path to compare.
+        reference_paths (List[str]): The list of reference paths.
+
+    Returns:
+        Union[str, None]: The most common path or None if no common path is found.
+    """
+    path_match_lst = [p for p in reference_paths if os.path.commonpath([path, p]) == p]
+    if len(path_match_lst) > 0:
+        return max(path_match_lst, key=len)
     else:
         return None
