@@ -5,31 +5,36 @@
 DatabaseAccess class deals with accessing the database
 """
 
-from pyiron_base.state.logger import logger
-from abc import ABC, abstractmethod
-from collections.abc import Iterable
-import warnings
-import numpy as np
-import re
 import os
+import re
+import warnings
 from datetime import datetime
-from pyiron_base.utils.deprecate import deprecate
+from queue import Empty as QueueEmpty
+from queue import SimpleQueue
+from threading import Lock, Thread
+from typing import List, Optional, Union
+
+import numpy as np
 import pandas
+from pyiron_snippets.deprecate import deprecate
+from pyiron_snippets.logger import logger
+from pyiron_snippets.retry import retry
 from sqlalchemy import (
-    create_engine,
     MetaData,
     Table,
-    text,
     and_,
+    create_engine,
     or_,
+    text,
 )
+from sqlalchemy.engine import Engine
+from sqlalchemy.exc import DatabaseError, OperationalError
 from sqlalchemy.pool import NullPool
 from sqlalchemy.sql import select
-from sqlalchemy.exc import OperationalError, DatabaseError
-from threading import Thread, Lock
-from queue import SimpleQueue, Empty as QueueEmpty
+
+from pyiron_base.database.interface import IsDatabase
+from pyiron_base.database.sqlcolumnlength import CHEMICALFORMULA_STR_LENGTH
 from pyiron_base.database.tables import get_historical_table
-from pyiron_base.utils.error import retry
 
 __author__ = "Murat Han Celik"
 __copyright__ = (
@@ -43,281 +48,6 @@ __status__ = "production"
 __date__ = "Sep 1, 2017"
 
 
-class IsDatabase(ABC):
-    """
-    Captures common interface for all database types in pyiron, e.g. SQL/SQLite/FileTable.
-    """
-
-    @abstractmethod
-    def _get_view_mode(self):
-        pass
-
-    @property
-    def view_mode(self):
-        """
-        Get view_mode - if view_moded is enable pyiron has read only access to the database.
-
-        Some implementations do not allow to set this value.
-
-        Returns:
-            bool: True when view_mode is enabled
-        """
-        return self._get_view_mode()
-
-    @property
-    @deprecate("use view_mode")
-    def viewer_mode(self):
-        return self.view_mode
-
-    viewer_mode.__doc__ = view_mode.__doc__
-
-    @abstractmethod
-    def _get_job_table(
-        self,
-        sql_query,
-        user,
-        project_path,
-        recursive=True,
-        columns=None,
-        element_lst=None,
-    ):
-        pass
-
-    @staticmethod
-    def _get_filtered_job_table(
-        df: pandas.DataFrame, **kwargs: dict
-    ) -> pandas.DataFrame:
-        """
-        Get a job table in a project based on matching values from any column in the project database
-
-        The values in `kwargs` can be wildcards, with the following special charaters:
-            - !value matches in the inverse of value
-            - *value matches anything that ends in value
-            - value* matches anything that starts with value
-            - *value* matches anything that contains value
-
-        Args:
-            df (pandas.DataFrame): DataFrame to be filtered
-            **kwargs (dict): Optional arguments for filtering with keys matching the project database column name
-                            (eg. status="finished")
-
-        Returns:
-            list: DataFrame containing filtered jobs
-        """
-        if len(kwargs) == 0 or df.empty:
-            return df
-        mask = np.ones_like(df.index, dtype=bool)
-        for key in kwargs.keys():
-            if key not in list(df.columns):
-                raise ValueError(
-                    f"Column name {key} does not exist in the project database!"
-                )
-        for key, val in kwargs.items():
-            invert = False
-            if isinstance(val, str) and val[0] == "!":
-                invert = True
-                val = val[1:]
-            if val is None:
-                update = df[key].isnull()
-            elif str(val).startswith("*") and str(val).endswith("*"):
-                update = df[key].str.contains(str(val).replace("*", ""))
-            elif str(val).endswith("*"):
-                update = df[key].str.startswith(str(val).replace("*", ""))
-            elif str(val).startswith("*"):
-                update = df[key].str.endswith(str(val).replace("*", ""))
-            else:
-                update = df[key] == val
-            if invert:
-                update = ~update
-            mask &= update
-        return df[mask]
-
-    def job_table(
-        self,
-        sql_query,
-        user,
-        project_path,
-        recursive=True,
-        columns=None,
-        all_columns=False,
-        sort_by="id",
-        max_colwidth=200,
-        full_table=False,
-        element_lst=None,
-        job_name_contains="",
-        **kwargs,
-    ):
-        """
-        Access the job_table.
-
-        Args:
-            sql_query (str): SQL query to enter a more specific request
-            user (str): username of the user whoes user space should be searched
-            project_path (str): root_path - this is in contrast to the project_path in GenericPath
-            recursive (bool): search subprojects [True/False]
-            columns (list): by default only the columns ['job', 'project', 'chemicalformula'] are selected, but the
-                            user can select a subset of ['id', 'status', 'chemicalformula', 'job', 'subjob', 'project',
-                            'projectpath', 'timestart', 'timestop', 'totalcputime', 'computer', 'hamilton', 'hamversion',
-                            'parentid', 'masterid']
-            all_columns (bool): Select all columns - this overwrites the columns option.
-            sort_by (str): Sort by a specific column
-            max_colwidth (int): set the column width
-            full_table (bool): Whether to show the entire pandas table
-            element_lst (list): list of elements required in the chemical formular - by default None
-            job_name_contains (str): (deprecated) A string which should be contained in every job_name
-            **kwargs (dict): Optional arguments for filtering with keys matching the project database column name
-                            (eg. status="finished"). Asterisk can be used to denote a wildcard, for zero or more
-                            instances of any character
-
-        Returns:
-            pandas.Dataframe: Return the result as a pandas.Dataframe object
-        """
-        if columns is None:
-            columns = ["job", "project", "chemicalformula"]
-        if all_columns:
-            columns = [
-                "id",
-                "status",
-                "chemicalformula",
-                "job",
-                "subjob",
-                "projectpath",
-                "project",
-                "timestart",
-                "timestop",
-                "totalcputime",
-                "computer",
-                "hamilton",
-                "hamversion",
-                "parentid",
-                "masterid",
-            ]
-        if sort_by not in columns:
-            columns = list(columns) + [sort_by]
-        if full_table:
-            pandas.set_option("display.max_rows", None)
-            pandas.set_option("display.max_columns", None)
-        else:
-            pandas.reset_option("display.max_rows")
-            pandas.reset_option("display.max_columns")
-        pandas.set_option("display.max_colwidth", max_colwidth)
-        df = self._get_job_table(
-            user=user,
-            sql_query=sql_query,
-            project_path=project_path,
-            recursive=recursive,
-            columns=columns,
-        )
-        if job_name_contains != "":
-            warnings.warn(
-                "`job_name_contains` is deprecated - use `job='*term*'` instead"
-            )
-            kwargs["job"] = "*{}*".format(job_name_contains)
-        df = self._get_filtered_job_table(df, **kwargs)
-        if sort_by is not None:
-            return df.sort_values(by=sort_by)
-        return df
-
-    @abstractmethod
-    def _get_table_headings(self, table_name=None):
-        pass
-
-    def item_update(self, par_dict, item_id):
-        if isinstance(item_id, Iterable):
-            return self._items_update(par_dict=par_dict, item_ids=item_id)
-        return self._item_update(par_dict=par_dict, item_id=item_id)
-
-    @abstractmethod
-    def _item_update(self, par_dict, item_id):
-        pass
-
-    def _items_update(self, par_dict, item_ids):
-        """
-        For now simply loops over all item_ids to call item_update,
-        but can be made more efficient.
-        Should be made an asbtract method when defined in inheriting classes
-
-        Args:
-            par_dict (_type_): _description_
-            item_ids (_type_): _description_
-        """
-        for i_id in item_ids:
-            self._item_update(par_dict=par_dict, item_id=i_id)
-
-    def set_job_status(self, status, job_id):
-        """
-        Set status of a job or multiple jobs if job_id is iterable.
-
-        Args:
-            status (str): status
-            job_id (int, Iterable): job id
-        """
-        if isinstance(job_id, Iterable):
-            return self._items_update(
-                par_dict={"status": status},
-                item_ids=job_id,
-            )
-        return self._item_update(
-            par_dict={"status": status},
-            item_id=job_id,
-        )
-
-    def get_table_headings(self, table_name=None):
-        """
-        Get column names; if given table_name can select one of multiple tables defined in the database, but subclasses
-        may ignore it
-
-        Args:
-            table_name (str): simple string of a table_name like: 'jobs_username'
-
-        Returns:
-            list: list of column names like:
-                ['id',
-                'parentid',
-                'masterid',
-                'projectpath',
-                'project',
-                'job',
-                'subjob',
-                'chemicalformula',
-                'status',
-                'hamilton',
-                'hamversion',
-                'username',
-                'computer',
-                'timestart',
-                'timestop',
-                'totalcputime']
-        """
-        return self._get_table_headings(table_name=table_name)
-
-    @deprecate("use get_table_headings()")
-    def get_db_columns(self):
-        """
-        Get column names
-
-        Returns:
-            list: list of column names like:
-                ['id',
-                'parentid',
-                'masterid',
-                'projectpath',
-                'project',
-                'job',
-                'subjob',
-                'chemicalformula',
-                'status',
-                'hamilton',
-                'hamversion',
-                'username',
-                'computer',
-                'timestart',
-                'timestop',
-                'totalcputime']
-        """
-        return self.get_table_headings()
-
-
 class ConnectionWatchDog(Thread):
     """
     Helper class that closes idle connections after a given timeout.
@@ -358,7 +88,7 @@ class ConnectionWatchDog(Thread):
         self._lock = lock
         self._timeout = timeout
 
-    def run(self):
+    def run(self) -> None:
         """
         Starts the watchdog.
         """
@@ -371,17 +101,17 @@ class ConnectionWatchDog(Thread):
                 with self._lock:
                     try:
                         self._conn.close()
-                    except:
+                    except (DatabaseError, OperationalError):
                         pass
                     break
 
-    def kick(self):
+    def kick(self) -> None:
         """
         Restarts the timeout.
         """
         self._queue.put(True)
 
-    def kill(self):
+    def kill(self) -> None:
         """
         Stop the watchdog and close the connection.
         """
@@ -390,7 +120,7 @@ class ConnectionWatchDog(Thread):
 
 
 class AutorestoredConnection:
-    def __init__(self, engine, timeout=60):
+    def __init__(self, engine: Engine, timeout: int = 60):
         self.engine = engine
         self._conn = None
         self._lock = Lock()
@@ -431,11 +161,11 @@ class AutorestoredConnection:
             delay=5,
         )
 
-    def close(self):
+    def close(self) -> None:
         if self._conn is not None:
             self._conn.close()
 
-    def commit(self):
+    def commit(self) -> None:
         if self._conn is not None:
             self._conn.commit()
 
@@ -454,7 +184,7 @@ class DatabaseAccess(IsDatabase):
     Murat Han Celik
     """
 
-    def __init__(self, connection_string, table_name, timeout=60):
+    def __init__(self, connection_string: str, table_name: str, timeout: int = 60):
         """
         Initialize the Database connection
 
@@ -487,9 +217,9 @@ class DatabaseAccess(IsDatabase):
         except Exception as except_msg:
             raise ValueError("Connection to database failed: " + str(except_msg))
 
-        self._chem_formula_lim_length = 50
+        self._chem_formula_lim_length = CHEMICALFORMULA_STR_LENGTH
 
-        def _create_table():
+        def _create_table() -> None:
             self.__reload_db()
             self.simulation_table = get_historical_table(
                 table_name=str(table_name), metadata=self.metadata, extend_existing=True
@@ -507,11 +237,11 @@ class DatabaseAccess(IsDatabase):
         )
         self._view_mode = False
 
-    def _get_view_mode(self):
+    def _get_view_mode(self) -> bool:
         return self._view_mode
 
     @IsDatabase.view_mode.setter
-    def view_mode(self, value):
+    def view_mode(self, value: bool) -> None:
         """
         Set view_mode - if view_mode is enable pyiron has read only access to the database.
 
@@ -525,19 +255,19 @@ class DatabaseAccess(IsDatabase):
 
     @IsDatabase.viewer_mode.setter
     @deprecate("use view_mode")
-    def viewer_mode(self, value):
+    def viewer_mode(self, value: bool) -> None:
         self.view_mode = value
 
     def _job_dict(
         self,
-        sql_query,
-        user,
-        project_path,
-        recursive,
-        job=None,
-        sub_job_name="%",
-        element_lst=None,
-    ):
+        sql_query: str,
+        user: str,
+        project_path: str,
+        recursive: bool,
+        job: Optional[str] = None,
+        sub_job_name: str = "%",
+        element_lst: List[str] = None,
+    ) -> List[dict]:
         """
         Internal function to access the database from the project directly.
 
@@ -619,13 +349,13 @@ class DatabaseAccess(IsDatabase):
 
     def _get_job_table(
         self,
-        sql_query,
-        user,
-        project_path,
-        recursive=True,
-        columns=None,
-        element_lst=None,
-    ):
+        sql_query: str,
+        user: str,
+        project_path: str,
+        recursive: bool = True,
+        columns: List[str] = None,
+        element_lst: List[str] = None,
+    ) -> pandas.DataFrame:
         job_dict = self._job_dict(
             sql_query=sql_query,
             user=user,
@@ -636,7 +366,7 @@ class DatabaseAccess(IsDatabase):
         return pandas.DataFrame(job_dict, columns=columns)
 
     # Internal functions
-    def __del__(self):
+    def __del__(self) -> None:
         """
         Close database connection
 
@@ -646,7 +376,7 @@ class DatabaseAccess(IsDatabase):
         if not self._keep_connection:
             self.conn.close()
 
-    def __reload_db(self):
+    def __reload_db(self) -> None:
         """
         Reload database
 
@@ -657,7 +387,7 @@ class DatabaseAccess(IsDatabase):
         self.metadata.reflect(bind=self._engine)
 
     @staticmethod
-    def regexp(expr, item):
+    def regexp(expr: str, item: str) -> Union[str, None]:
         """
         Regex function for SQLite
         Args:
@@ -677,7 +407,7 @@ class DatabaseAccess(IsDatabase):
             return reg.search(item) is not None
 
     # Table functions
-    def _get_table_headings(self, table_name=None):
+    def _get_table_headings(self, table_name: Optional[str] = None) -> List[str]:
         """
         Get column names
 
@@ -716,7 +446,9 @@ class DatabaseAccess(IsDatabase):
             raise ValueError(str(table_name) + " does not exist")
         return [column.name for column in iter(simulation_list.columns)]
 
-    def add_column(self, col_name, col_type):
+    def add_column(
+        self, col_name: Union[str, List[str]], col_type: Union[str, List[str]]
+    ) -> None:
         """
         Add an additional column - required for modification on the database
 
@@ -742,7 +474,9 @@ class DatabaseAccess(IsDatabase):
         else:
             raise PermissionError("Not avilable in viewer mode.")
 
-    def change_column_type(self, col_name, col_type):
+    def change_column_type(
+        self, col_name: Union[str, List[str]], col_type: Union[str, List[str]]
+    ) -> None:
         """
         Modify data type of an existing column - required for modification on the database
 
@@ -768,7 +502,9 @@ class DatabaseAccess(IsDatabase):
         else:
             raise PermissionError("Not avilable in viewer mode.")
 
-    def get_items_sql(self, where_condition=None, sql_statement=None):
+    def get_items_sql(
+        self, where_condition: Optional[str] = None, sql_statement: Optional[str] = None
+    ) -> List[dict]:
         """
         Submit an SQL query to the database
 
@@ -860,12 +596,12 @@ class DatabaseAccess(IsDatabase):
             output_list += [dict(zip(col.keys(), tmp_values))]
         return output_list
 
-    def _check_chem_formula_length(self, par_dict):
+    def _check_chem_formula_length(self, par_dict: dict) -> dict:
         """
         performs a check whether the length of chemical formula exceeds the defined limit
-        args:
-        par_dict(dict): dictionary of the parameter
-        limit(int): the limit for the length of checmical formular
+
+        Args:
+            par_dict(dict): dictionary of the parameters to be checked
         """
         key_limited = "ChemicalFormula"
         if (
@@ -876,8 +612,24 @@ class DatabaseAccess(IsDatabase):
             par_dict[key_limited] = "OVERFLOW_ERROR"
         return par_dict
 
+    def _check_duplidates(self, par_dict: dict) -> bool:
+        """
+        Check for duplicates in the database
+
+        Args:
+            par_dict (dict): Dictionary with the item values and column names as keys
+        """
+        return (
+            len(
+                self.get_items_dict(
+                    {"job": par_dict["job"], "project": par_dict["project"]}
+                )
+            )
+            > 0
+        )
+
     # Item functions
-    def add_item_dict(self, par_dict):
+    def add_item_dict(self, par_dict: dict, check_duplicates: bool = False) -> int:
         """
         Create a new database item
 
@@ -898,12 +650,16 @@ class DatabaseAccess(IsDatabase):
                              'timestop': datetime(2016, 5, 2, 11, 31, 4, 371165),
                              'totalcputime': 0.117788,
                              'username': 'Test'}
+            check_duplicates (bool): Check for duplicate entries in the database
 
         Returns:
             int: Database ID of the item created as an int, like: 3
         """
         if not self._view_mode:
             try:
+                if check_duplicates and self._check_duplidates(par_dict):
+                    warnings.warn(f"Duplicate entry found in database: {par_dict}")
+                    return None
                 par_dict = self._check_chem_formula_length(par_dict)
                 par_dict = dict(
                     (key.lower(), value) for key, value in par_dict.items()
@@ -920,7 +676,7 @@ class DatabaseAccess(IsDatabase):
         else:
             raise PermissionError("Not avilable in viewer mode.")
 
-    def __get_items(self, col_name, var):
+    def __get_items(self, col_name: str, var: Union[str, int]) -> List[dict]:
         """
         Get multiple items from the database
 
@@ -949,7 +705,7 @@ class DatabaseAccess(IsDatabase):
                       'username': u'Test'}]
         """
         try:
-            if type(var) is list:
+            if isinstance(var, list):
                 var = var[-1]
             query = select(self.simulation_table).where(
                 self.simulation_table.c[str(col_name)] == var
@@ -970,7 +726,7 @@ class DatabaseAccess(IsDatabase):
             self.conn.close()
         return [dict(zip(col._mapping.keys(), col._mapping.values())) for col in row]
 
-    def _item_update(self, par_dict, item_id):
+    def _item_update(self, par_dict: dict, item_id: int) -> None:
         """
         Modify Item in database
 
@@ -1011,7 +767,7 @@ class DatabaseAccess(IsDatabase):
         else:
             raise PermissionError("Not avilable in viewer mode.")
 
-    def delete_item(self, item_id):
+    def delete_item(self, item_id: int) -> None:
         """
         Delete Item from database
 
@@ -1021,20 +777,43 @@ class DatabaseAccess(IsDatabase):
         Returns:
 
         """
-        if not self._view_mode:
-            self.conn.execute(
-                self.simulation_table.delete().where(
-                    self.simulation_table.c["id"] == int(item_id)
-                )
-            )
-            self.conn.commit()
-            if not self._keep_connection:
-                self.conn.close()
-        else:
+        if self._view_mode:
             raise PermissionError("Not avilable in viewer mode.")
 
+        res = self.conn.execute(
+            self.simulation_table.delete().where(
+                self.simulation_table.c["id"] == int(item_id)
+            )
+        )
+        if res.rowcount == 0:
+            raise RuntimeError(f"Failed to delete job ({item_id}) from database!")
+        self.conn.commit()
+
+        if not self._keep_connection:
+            self.conn.close()
+
+    # IsDatabase impl'
+    def _get_jobs(
+        self,
+        sql_query: str,
+        user: str,
+        project_path: str,
+        recursive: bool = True,
+        columns: Optional[List[str]] = None,
+    ) -> List[dict]:
+        df = self.job_table(
+            sql_query=sql_query,
+            user=user,
+            project_path=project_path,
+            recursive=recursive,
+            columns=columns,
+        )
+        if len(df) == 0:
+            return {key: list() for key in columns}
+        return df.to_dict(orient="list")
+
     # Shortcut
-    def get_item_by_id(self, item_id):
+    def get_item_by_id(self, item_id: int) -> dict:
         """
         Get item from database by searching for a specific item Id.
 
@@ -1081,7 +860,7 @@ class DatabaseAccess(IsDatabase):
         else:
             raise TypeError("THE SQL database ID has to be an integer.")
 
-    def query_for_element(self, element):
+    def query_for_element(self, element: str) -> Union[bool, str]:
         return or_(
             *[
                 self.simulation_table.c["chemicalformula"].like(
@@ -1091,7 +870,9 @@ class DatabaseAccess(IsDatabase):
             ]
         )
 
-    def get_items_dict(self, item_dict, return_all_columns=True):
+    def get_items_dict(
+        self, item_dict: dict, return_all_columns: bool = True
+    ) -> List[dict]:
         """
 
         Args:
@@ -1161,9 +942,11 @@ class DatabaseAccess(IsDatabase):
                 ]
             elif isinstance(value, list):
                 or_statement = [
-                    self.simulation_table.c[str(key)] == element
-                    if "%" not in element
-                    else self.simulation_table.c[str(key)].like(element)
+                    (
+                        self.simulation_table.c[str(key)] == element
+                        if "%" not in element
+                        else self.simulation_table.c[str(key)].like(element)
+                    )
                     for element in value
                 ]
                 # here we wrap the given values in an sqlalchemy-type or_statement
@@ -1196,13 +979,13 @@ class DatabaseAccess(IsDatabase):
             self.conn.close()
         return [dict(zip(col._mapping.keys(), col._mapping.values())) for col in row]
 
-    def get_job_status(self, job_id):
+    def get_job_status(self, job_id: int) -> Union[str, None]:
         try:
             return self.get_item_by_id(item_id=job_id)["status"]
         except KeyError:
             return None
 
-    def get_job_working_directory(self, job_id):
+    def get_job_working_directory(self, job_id: int) -> Union[str, None]:
         try:
             db_entry = self.get_item_by_id(job_id)
             if db_entry:
