@@ -5,8 +5,12 @@
 The Jobtable module provides a set of top level functions to interact with the database.
 """
 
+from typing import List, Optional, Union
+
 import numpy as np
+
 from pyiron_base.database.filetable import FileTable
+from pyiron_base.database.generic import DatabaseAccess
 
 __author__ = "Jan Janssen"
 __copyright__ = (
@@ -20,69 +24,14 @@ __status__ = "production"
 __date__ = "Sep 1, 2017"
 
 
-def get_jobs(database, sql_query, user, project_path, recursive=True, columns=None):
-    """
-    Internal function to return the jobs as dictionary rather than a pandas.Dataframe
-
-    Args:
-        database (DatabaseAccess): Database object
-        sql_query (str): SQL query to enter a more specific request
-        user (str): username of the user whoes user space should be searched
-        project_path (str): root_path - this is in contrast to the project_path in GenericPath
-        recursive (bool): search subprojects [True/False]
-        columns (list): by default only the columns ['id', 'project'] are selected, but the user can select a subset
-                        of ['id', 'status', 'chemicalformula', 'job', 'subjob', 'project', 'projectpath', 'timestart',
-                        'timestop', 'totalcputime', 'computer', 'hamilton', 'hamversion', 'parentid', 'masterid']
-
-    Returns:
-        dict: columns are used as keys and point to a list of the corresponding values
-    """
-    if not isinstance(database, FileTable):
-        if columns is None:
-            columns = ["id", "project"]
-        df = database.job_table(
-            sql_query=sql_query,
-            user=user,
-            project_path=project_path,
-            recursive=recursive,
-            columns=columns,
-        )
-        if len(df) == 0:
-            return {key: list() for key in columns}
-        return df.to_dict(orient="list")
-    else:
-        return database.get_jobs(
-            project=project_path, recursive=recursive, columns=columns
-        )
-
-
-def get_job_ids(database, sql_query, user, project_path, recursive=True):
-    """
-    Return the job IDs matching a specific query
-
-    Args:
-        database (DatabaseAccess): Database object
-        sql_query (str): SQL query to enter a more specific request
-        user (str): username of the user whoes user space should be searched
-        project_path (str): root_path - this is in contrast to the project_path in GenericPath
-        recursive (bool): search subprojects [True/False]
-
-    Returns:
-        list: a list of job IDs
-    """
-    if not isinstance(database, FileTable):
-        return get_jobs(
-            database=database,
-            sql_query=sql_query,
-            user=user,
-            project_path=project_path,
-            recursive=recursive,
-        )["id"]
-    else:
-        return database.get_job_ids(project=project_path, recursive=recursive)
-
-
-def get_child_ids(database, sql_query, user, project_path, job_specifier, status=None):
+def get_child_ids(
+    database: Union[FileTable, DatabaseAccess],
+    sql_query: str,
+    user: str,
+    project_path: str,
+    job_specifier: str,
+    status: Optional[str] = None,
+) -> List[dict]:
     """
     Get the childs for a specific job
 
@@ -117,7 +66,13 @@ def get_child_ids(database, sql_query, user, project_path, job_specifier, status
         return database.get_child_ids(job_specifier=job_specifier, project=project_path)
 
 
-def get_job_id(database, sql_query, user, project_path, job_specifier):
+def get_job_id(
+    database: Union[FileTable, DatabaseAccess],
+    sql_query: str,
+    user: str,
+    project_path: str,
+    job_specifier: str,
+) -> Union[int, None]:
     """
     get the job_id for job named job_name in the local project path from database
 
@@ -164,7 +119,14 @@ def get_job_id(database, sql_query, user, project_path, job_specifier):
         return database.get_job_id(job_specifier=job_specifier, project=project_path)
 
 
-def set_job_status(database, sql_query, user, project_path, job_specifier, status):
+def set_job_status(
+    database: Union[FileTable, DatabaseAccess],
+    sql_query: str,
+    user: str,
+    project_path: str,
+    job_specifier: Union[str, int],
+    status: str,
+) -> None:
     """
     Set the status of a particular job
 
@@ -190,7 +152,13 @@ def set_job_status(database, sql_query, user, project_path, job_specifier, statu
     )
 
 
-def get_job_status(database, sql_query, user, project_path, job_specifier):
+def get_job_status(
+    database: Union[FileTable, DatabaseAccess],
+    sql_query: str,
+    user: str,
+    project_path: str,
+    job_specifier: Union[str, int],
+) -> str:
     """
     Get the status of a particular job
 
@@ -217,7 +185,13 @@ def get_job_status(database, sql_query, user, project_path, job_specifier):
     )
 
 
-def get_job_working_directory(database, sql_query, user, project_path, job_specifier):
+def get_job_working_directory(
+    database: Union[FileTable, DatabaseAccess],
+    sql_query: str,
+    user: str,
+    project_path: str,
+    job_specifier: Union[str, int],
+) -> str:
     """
     Get the working directory of a particular job
 
