@@ -11,26 +11,32 @@ import os
 import posixpath
 import shutil
 import warnings
+from typing import Any, Generator, List, Optional, Union
+
+from pyiron_snippets.deprecate import deprecate
 
 from pyiron_base.interfaces.has_groups import HasGroups
-from pyiron_base.storage.hdfio import ProjectHDFio
+from pyiron_base.jobs.job.extension.files import FileBrowser
 from pyiron_base.jobs.job.util import (
-    _get_project_for_copy,
     _copy_database_entry,
     _copy_to_delete_existing,
-    _rename_job,
+    _get_project_for_copy,
     _get_safe_job_name,
-    _job_is_archived,
     _job_archive,
-    _job_unarchive,
-    _job_is_compressed,
     _job_compress,
     _job_decompress,
     _job_delete_files,
     _job_delete_hdf,
+    _job_is_archived,
+    _job_is_compressed,
+    _job_list_files,
+    _job_read_file,
     _job_remove_folder,
+    _job_unarchive,
+    _rename_job,
 )
 from pyiron_base.state import state
+from pyiron_base.storage.hdfio import ProjectHDFio
 
 __author__ = "Jan Janssen"
 __copyright__ = (
@@ -103,6 +109,137 @@ _doc_str_job_core_attr = """\
 """
 
 
+def recursive_load_from_hdf(project_hdf5: ProjectHDFio, item: str):
+    """
+    Load given item from HDF, but check also for DataContainer along the way.
+
+    If `item` exists as is in HDF, return it, otherwise break it up along every slash and try to load a
+    :class:`~.DataContainer` and then try to index with the remainder of the path, i.e.
+
+    >>> recursive_load_from_hdf(hdf, 'my/path/to/value')
+
+    is equivalent to one of (in this order)
+
+    >>> hdf['my/path/to'].to_object()['value']
+    >>> hdf['my/path'].to_object()['to/value']
+    >>> hdf['my'].to_object()['path/to/value']
+
+    in case
+
+    >>> hdf['/my/path/to/value']
+
+    doesn't exist.
+
+    Args:
+        project_hdf5 (ProjectHDFio): HDF file to access
+        item (str): path to value, may contain `/`
+
+    Returns:
+        object: whatever was found in the HDF file
+        None: if nothing was found in the HDF file
+    """
+
+    def successive_path_splits(name_lst: list) -> Generator:
+        """
+        Yield successive split/joins of a path, i.e.
+        /a/b/c/d
+        gives
+        /a/b/c, d
+        /a/b, c/d
+        /a, b/c/d
+        """
+        for i in range(1, len(name_lst)):
+            # where we are looking for the data container
+            container_path = "/".join(name_lst[:-i])
+            # where we are looking for data in the container
+            data_path = "/".join(name_lst[-i:])
+            yield container_path, data_path
+
+    try:
+        group = project_hdf5[item]
+        if (
+            isinstance(group, ProjectHDFio)
+            and "NAME" in group
+            and group["NAME"] == "DataContainer"
+        ):
+            return group.to_object(lazy=True)
+        else:
+            return group
+    except ValueError:
+        pass
+
+    name_lst = item.split("/")
+
+    for container_path, data_path in successive_path_splits(name_lst):
+        try:
+            group = project_hdf5[container_path]
+            if (
+                isinstance(group, ProjectHDFio)
+                and "NAME" in group
+                and group["NAME"] == "DataContainer"
+            ):
+                return group.to_object(lazy=True)[data_path]
+        except (ValueError, IndexError, KeyError):
+            # either group does not contain a data container or it is does, but it does not have the path we're
+            # looking for
+            pass
+
+
+class DatabaseProperties(object):
+    """
+    Access the database entry of the job
+    """
+
+    def __init__(self, job_dict=None):
+        self._job_dict = job_dict
+
+    def __bool__(self):
+        return self._job_dict is not None
+
+    def __dir__(self):
+        return list(self._job_dict.keys())
+
+    def __getattr__(self, name):
+        if name in self._job_dict.keys():
+            return self._job_dict[name]
+        else:
+            raise AttributeError(name)
+
+    def __repr__(self):
+        return f"{self.__class__.__name__}({repr(self._job_dict)})"
+
+
+class HDF5Content(object):
+    """
+    Access the HDF5 file of the job
+    """
+
+    def __init__(self, project_hdf5):
+        self._project_hdf5 = project_hdf5
+
+    def __getattr__(self, name):
+        try:
+            return self[name]
+        except KeyError:
+            raise AttributeError(name) from None
+
+    def __getitem__(self, item):
+        value = recursive_load_from_hdf(self._project_hdf5, item)
+        if value is not None:
+            return value
+
+        if item in self._project_hdf5.list_groups():
+            return HDF5Content(self._project_hdf5[item])
+        else:
+            raise KeyError(item)
+
+    def __dir__(self):
+        return self._project_hdf5.list_nodes() + self._project_hdf5.list_groups()
+
+    def __repr__(self):
+        return self._project_hdf5.__repr__()
+
+
 class JobCore(HasGroups):
     __doc__ = (
         """
@@ -118,7 +255,7 @@ class JobCore(HasGroups):
         + _doc_str_job_core_attr
     )
 
-    def __init__(self, project, job_name):
+    def __init__(self, project: ProjectHDFio, job_name: str):
         job_name = _get_safe_job_name(job_name)
         self._name = job_name
         self._hdf5 = project.open(self._name)
@@ -129,13 +266,21 @@ class JobCore(HasGroups):
         self._import_directory = None
         self._database_property = DatabaseProperties()
         self._hdf5_content = HDF5Content(project_hdf5=self._hdf5)
+        self._files_to_remove = list()
+        self._files_to_compress = list()
 
     @property
-    def content(self):
+    def content(self) -> HDF5Content:
         return self._hdf5_content
 
     @property
-    def job_name(self):
+    def files(self) -> FileBrowser:
+        return FileBrowser(working_directory=self.working_directory)
+
+    files.__doc__ = FileBrowser.__doc__
+
+    @property
+    def job_name(self) -> str:
         """
         Get name of the job, which has to be unique within the project
 
@@ -145,7 +290,7 @@ class JobCore(HasGroups):
         return self.name
 
     @job_name.setter
-    def job_name(self, new_job_name):
+    def job_name(self, new_job_name: str) -> None:
         """
         Set name of the job, which has to be unique within the project. When changing the job_name this also moves the
         HDF5 file as the name of the HDF5 file is the job_name plus the extension *.h5
@@ -156,7 +301,7 @@ class JobCore(HasGroups):
         self.name = new_job_name
 
     @property
-    def name(self):
+    def name(self) -> str:
         """
         Get name of the job, which has to be unique within the project
 
@@ -166,7 +311,7 @@ class JobCore(HasGroups):
         return self._name
 
     @name.setter
-    def name(self, new_job_name):
+    def name(self, new_job_name: str) -> None:
         """
         Set name of the job, which has to be unique within the project. When changing the job_name this also moves the
         HDF5 file as the name of the HDF5 file is the job_name plus the extension *.h5
@@ -177,7 +322,7 @@ class JobCore(HasGroups):
         _rename_job(job=self, new_job_name=new_job_name)
 
     @property
-    def status(self):
+    def status(self) -> str:
         """
         Execution status of the job, can be one of the following [initialized, appended, created, submitted, running,
                                                                   aborted, collect, suspended, refresh, busy, finished]
@@ -188,7 +333,7 @@ class JobCore(HasGroups):
         return self._status
 
     @property
-    def job_id(self):
+    def job_id(self) -> int:
         """
         Unique id to identify the job in the pyiron database
 
@@ -200,7 +345,7 @@ class JobCore(HasGroups):
         return self._job_id
 
     @property
-    def id(self):
+    def id(self) -> int:
         """
         Unique id to identify the job in the pyiron database - use self.job_id instead
 
@@ -210,7 +355,7 @@ class JobCore(HasGroups):
         return self.job_id
 
     @property
-    def database_entry(self):
+    def database_entry(self) -> DatabaseProperties:
         if not bool(self._database_property):
             self._database_property = DatabaseProperties(
                 job_dict=self.project.db.get_item_by_id(self.job_id)
@@ -218,7 +363,7 @@ class JobCore(HasGroups):
         return self._database_property
 
     @property
-    def parent_id(self):
+    def parent_id(self) -> int:
         """
         Get job id of the predecessor job - the job which was executed before the current one in the current job series
 
@@ -230,7 +375,7 @@ class JobCore(HasGroups):
         return self._parent_id
 
     @parent_id.setter
-    def parent_id(self, parent_id):
+    def parent_id(self, parent_id: int) -> None:
         """
         Set job id of the predecessor job - the job which was executed before the current one in the current job series
 
@@ -242,7 +387,7 @@ class JobCore(HasGroups):
         self._parent_id = parent_id
 
     @property
-    def master_id(self):
+    def master_id(self) -> int:
         """
         Get job id of the master job - a meta job which groups a series of jobs, which are executed either in parallel
         or in serial.
@@ -255,7 +400,7 @@ class JobCore(HasGroups):
         return self._master_id
 
     @master_id.setter
-    def master_id(self, master_id):
+    def master_id(self, master_id: int) -> None:
         """
         Set job id of the master job - a meta job which groups a series of jobs, which are executed either in parallel
         or in serial.
@@ -268,7 +413,7 @@ class JobCore(HasGroups):
         self._master_id = master_id
 
     @property
-    def child_ids(self):
+    def child_ids(self) -> list:
         """
         list of child job ids - only meta jobs have child jobs - jobs which list the meta job as their master
 
@@ -280,7 +425,7 @@ class JobCore(HasGroups):
         )
 
     @property
-    def project_hdf5(self):
+    def project_hdf5(self) -> ProjectHDFio:
         """
         Get the ProjectHDFio instance which points to the HDF5 file the job is stored in
 
@@ -290,7 +435,7 @@ class JobCore(HasGroups):
         return self._hdf5
 
     @project_hdf5.setter
-    def project_hdf5(self, project):
+    def project_hdf5(self, project: ProjectHDFio) -> None:
         """
         Set the ProjectHDFio instance which points to the HDF5 file the job is stored in
 
@@ -299,7 +444,15 @@ class JobCore(HasGroups):
         """
         self._hdf5 = project.copy()
 
-    def relocate_hdf5(self, h5_path=None):
+    @property
+    def files_to_compress(self) -> list:
+        return self._files_to_compress
+
+    @property
+    def files_to_remove(self) -> list:
+        return self._files_to_remove
+
+    def relocate_hdf5(self, h5_path: Optional[str] = None):
         """
         Relocate the hdf file. This function is needed when the child job is
         spawned by a parent job (cf. pyiron_base.jobs.master.generic)
@@ -312,7 +465,7 @@ class JobCore(HasGroups):
         )
 
     @property
-    def project(self):
+    def project(self) -> "pyiron_base.project.generic.Project":
         """
         Project instance the jobs is located in
 
@@ -322,7 +475,7 @@ class JobCore(HasGroups):
         return self._hdf5.project
 
     @property
-    def job_info_str(self):
+    def job_info_str(self) -> str:
         """
         Short string to describe the job by it is job_name and job ID - mainly used for logging
 
@@ -332,7 +485,7 @@ class JobCore(HasGroups):
         return "job: {0} id: {1}".format(self._name, self.job_id)
 
     @property
-    def working_directory(self):
+    def working_directory(self) -> str:
         """
         working directory of the job is executed in - outside the HDF5 file
 
@@ -342,7 +495,7 @@ class JobCore(HasGroups):
         return self.project_hdf5.working_directory
 
     @property
-    def path(self):
+    def path(self) -> str:
         """
         Absolute path of the HDF5 group starting from the system root - combination of the absolute system path plus the
         absolute path inside the HDF5 file starting from the root group.
@@ -352,7 +505,13 @@ class JobCore(HasGroups):
         """
         return self.project_hdf5.path
 
-    def check_if_job_exists(self, job_name=None, project=None):
+    def check_if_job_exists(
+        self,
+        job_name: Optional[str] = None,
+        project: Optional[
+            Union[ProjectHDFio, "pyiron_base.project.generic.Project"]
+        ] = None,
+    ):
         """
         Check if a job already exists in an specific project.
 
@@ -384,13 +543,13 @@ class JobCore(HasGroups):
         else:
             return False
 
-    def show_hdf(self):
+    def show_hdf(self) -> None:
         """
         Iterating over the HDF5 datastructure and generating a human readable graph.
         """
         self.project_hdf5.show_hdf()
 
-    def get_from_table(self, path, name):
+    def get_from_table(self, path: str, name: str) -> Union[dict, list, float, int]:
         """
         Get a specific value from a pandas.Dataframe
 
@@ -403,7 +562,7 @@ class JobCore(HasGroups):
         """
         return self.project_hdf5.get_from_table(path, name)
 
-    def remove(self, _protect_childs=True):
+    def remove(self, _protect_childs: bool = True) -> None:
         """
         Remove the job - this removes the HDF5 file, all data stored in the HDF5 file an the corresponding database entry.
 
@@ -422,7 +581,7 @@ class JobCore(HasGroups):
                     )
                     raise ValueError("Child jobs are protected and cannot be deleted!")
             for job_id in self.child_ids:
-                job = self.project.load(job_id, convert_to_object=False)
+                job = self.project.inspect(job_id)
                 if len(job.child_ids) > 0:
                     job.remove(_protect_childs=False)
                 else:
@@ -431,7 +590,7 @@ class JobCore(HasGroups):
         # After all children are deleted, remove the job itself.
         self.remove_child()
 
-    def remove_child(self):
+    def remove_child(self) -> None:
         """
         internal function to remove command that removes also child jobs.
         Do never use this command, since it will destroy the integrity of your project.
@@ -480,7 +639,9 @@ class JobCore(HasGroups):
         if self.job_id is not None:
             self.project.db.delete_item(self.job_id)
 
-    def to_object(self, object_type=None, **qwargs):
+    def to_object(
+        self, object_type: Optional[str] = None, **qwargs
+    ) -> "pyiron_base.job.generic.GenericJob":
         """
         Load the full pyiron object from an HDF5 file
 
@@ -499,7 +660,7 @@ class JobCore(HasGroups):
             )
         return self.project_hdf5.to_object(object_type, **qwargs)
 
-    def get(self, name, default=None):
+    def get(self, name: str, default: Optional[Any] = None) -> Any:
         """
         Internal wrapper function for __getitem__() - self[name]
 
@@ -520,7 +681,9 @@ class JobCore(HasGroups):
                 return default
             raise
 
-    def load(self, job_specifier, convert_to_object=True):
+    def load(
+        self, job_specifier: Union[str, int], convert_to_object: bool = True
+    ) -> Union["pyiron_base.job.generic.GenericJob", "JobCore"]:
         """
         Load an existing pyiron object - most commonly a job - from the database
 
@@ -537,7 +700,7 @@ class JobCore(HasGroups):
             job_specifier=job_specifier, convert_to_object=convert_to_object
         )
 
-    def inspect(self, job_specifier):
+    def inspect(self, job_specifier: Union[str, int]) -> "JobCore":
         """
         Inspect an existing pyiron object - most commonly a job - from the database
 
@@ -547,9 +710,9 @@ class JobCore(HasGroups):
         Returns:
             JobCore: Access to the HDF5 object - not a GenericJob object - use load() instead.
         """
-        return self.project.load(job_specifier=job_specifier, convert_to_object=False)
+        return self.project.inspect(job_specifier=job_specifier)
 
-    def is_master_id(self, job_id):
+    def is_master_id(self, job_id: int) -> bool:
         """
         Check if the job ID job_id is the master ID for any child job
 
@@ -571,7 +734,9 @@ class JobCore(HasGroups):
             > 0
         )
 
-    def get_job_id(self, job_specifier=None):
+    def get_job_id(
+        self, job_specifier: Optional[Union[str, int]] = None
+    ) -> Union[int, None]:
         """
         get the job_id for job named job_name in the local project path from database
 
@@ -595,7 +760,8 @@ class JobCore(HasGroups):
             return response[-1]["id"]
         return None
 
-    def list_files(self):
+    @deprecate("use job.files.list()")
+    def list_files(self) -> list:
         """
         List files inside the working directory
 
@@ -605,29 +771,24 @@ class JobCore(HasGroups):
         Returns:
             list: list of file names
         """
-        if os.path.isdir(self.working_directory):
-            return os.listdir(self.working_directory)
-        return []
+        return _job_list_files(self)
 
-    def list_childs(self):
+    def list_childs(self) -> list:
         """
         List child jobs as JobPath objects - not loading the full GenericJob objects for each child
 
         Returns:
             list: list of child jobs
         """
-        return [
-            self.project.load(child_id, convert_to_object=False).job_name
-            for child_id in self.child_ids
-        ]
+        return [self.project.inspect(child_id).job_name for child_id in self.child_ids]
 
-    def _list_groups(self):
+    def _list_groups(self) -> list:
         return self.project_hdf5.list_groups() + self._list_ext_childs()
 
-    def _list_nodes(self):
+    def _list_nodes(self) -> list:
         return self.project_hdf5.list_nodes()
 
-    def _list_all(self):
+    def _list_all(self) -> dict:
         """
         List all groups and nodes of the HDF5 file - where groups are equivalent to directories and nodes to files.
 
@@ -638,7 +799,7 @@ class JobCore(HasGroups):
         h5_dict["groups"] += self._list_ext_childs()
         return h5_dict
 
-    def copy(self):
+    def copy(self) -> "JobCore":
         """
         Copy the JobCore object which links to the HDF5 file
 
@@ -651,12 +812,14 @@ class JobCore(HasGroups):
 
     def _internal_copy_to(
         self,
-        project=None,
-        new_job_name=None,
-        new_database_entry=True,
-        copy_files=True,
-        delete_existing_job=False,
-    ):
+        project: Optional[
+            Union["JobCore", ProjectHDFio, "pyiron_base.project.generic.Project"]
+        ] = None,
+        new_job_name: Optional[str] = None,
+        new_database_entry: bool = True,
+        copy_files: bool = True,
+        delete_existing_job: bool = False,
+    ) -> "JobCore":
         """
         Internal helper function for copy_to() which returns more
 
@@ -731,12 +894,12 @@ class JobCore(HasGroups):
 
     def copy_to(
         self,
-        project,
-        new_job_name=None,
-        input_only=False,
-        new_database_entry=True,
-        copy_files=True,
-    ):
+        project: Union["JobCore", ProjectHDFio, "pyiron_base.project.generic.Project"],
+        new_job_name: Optional[str] = None,
+        input_only: bool = False,
+        new_database_entry: bool = True,
+        copy_files: bool = True,
+    ) -> "JobCore":
         """
         Copy the content of the job including the HDF5 file to a new location
 
@@ -778,15 +941,12 @@ class JobCore(HasGroups):
             new_job_core._status = "initialized"
         return new_job_core
 
-    def move_to(self, project):
+    def move_to(self, project: ProjectHDFio) -> None:
         """
         Move the content of the job including the HDF5 file to a new location
 
         Args:
             project (ProjectHDFio): project to move the job to
-
-        Returns:
-            JobCore: JobCore object pointing to the new location.
         """
         delete_hdf5_after_copy = False
         old_working_directory = self.working_directory
@@ -817,7 +977,7 @@ class JobCore(HasGroups):
             shutil.rmtree(old_working_directory)
             os.rmdir("/".join(old_working_directory.split("/")[:-1]))
 
-    def rename(self, new_job_name):
+    def rename(self, new_job_name: str) -> None:
         """
         Rename the job - by changing the job name
 
@@ -826,7 +986,7 @@ class JobCore(HasGroups):
         """
         self.job_name = new_job_name
 
-    def reset_job_id(self, job_id=None):
+    def reset_job_id(self, job_id: Optional[int] = None) -> None:
         """
         The reset_job_id function has to be implemented by the derived classes - usually the GenericJob class
 
@@ -838,13 +998,15 @@ class JobCore(HasGroups):
             job_id = int(job_id)
         self._job_id = job_id
 
-    def save(self):
+    def save(self) -> None:
         """
         The save function has to be implemented by the derived classes - usually the GenericJob class
         """
         raise NotImplementedError("save() should be implemented in the derived class")
 
-    def to_hdf(self, hdf=None, group_name="group"):
+    def to_hdf(
+        self, hdf: Optional[ProjectHDFio] = None, group_name: str = "group"
+    ) -> None:
         """
         Store object in hdf5 format - The function has to be implemented by the derived classes
         - usually the GenericJob class
@@ -855,7 +1017,9 @@ class JobCore(HasGroups):
         """
         raise NotImplementedError("to_hdf() should be implemented in the derived class")
 
-    def from_hdf(self, hdf=None, group_name="group"):
+    def from_hdf(
+        self, hdf: Optional[ProjectHDFio] = None, group_name: str = "group"
+    ) -> None:
         """
         Restore object from hdf5 format - The function has to be implemented by the derived classes
         - usually the GenericJob class
@@ -868,7 +1032,7 @@ class JobCore(HasGroups):
             "from_hdf() should be implemented in the derived class"
         )
 
-    def __del__(self):
+    def __del__(self) -> None:
         """
         The delete function is just implemented for compatibilty
         """
@@ -879,7 +1043,11 @@ class JobCore(HasGroups):
         del self._master_id
         del self._status
 
-    def __getitem__(self, item):
+    @deprecate(
+        "Use job.output for results, job.files to access files; job.content to access HDF storage and "
+        "job.child_project to access children of master jobs."
+    )
+    def __getitem__(self, item: str) -> Any:
         """
         Get/read data from the HDF5 file, child jobs or access log files.
 
@@ -899,58 +1067,26 @@ class JobCore(HasGroups):
         Returns:
             dict, list, float, int, :class:`.DataContainer`, None: data or data object; if nothing is found None is returned
         """
-
-        if item in self.list_files():
-            file_name = posixpath.join(self.working_directory, "{}".format(item))
-            with open(file_name) as f:
-                return f.readlines()
-
         # first try to access HDF5 directly to make the common case fast
-        try:
-            group = self._hdf5[item]
-            if (
-                isinstance(group, ProjectHDFio)
-                and "NAME" in group
-                and group["NAME"] == "DataContainer"
-            ):
-                return group.to_object(lazy=True)
-            else:
-                return group
-        except ValueError:
-            pass
+        value = recursive_load_from_hdf(self._hdf5, item)
+        if value is not None:
+            return value
+
+        # only try to read files when no slashes are present:
+        # downstream code will often do something like job['path/to/output'] to check if certain values exist and branch
+        # on that.  In cases where they don't exists this would then trigger us to decompress the job files in memory on
+        # every check which slows down things a lot.  Generally these value checks will be of the form output/.../...
+        # i.e. contain slashes and file access tend to be just the file name without slashes, so I separate those cases
+        # here like this.  In those cases where we actually have sub directories in the job folders we can beef up the
+        # file browser.
+        if "/" not in item and item in self.files.list():
+            warnings.warn(
+                "Using __getitem__ on a job to access files in deprecated: use job.files instead!",
+                category=DeprecationWarning,
+            )
+            return _job_read_file(self, item)
 
         name_lst = item.split("/")
-
-        def successive_path_splits(name_lst):
-            """
-            Yield successive split/joins of a path, i.e.
-            /a/b/c/d
-            gives
-            /a/b/c, d
-            /a/b, c/d
-            /a, b/c/d
-            """
-            for i in range(1, len(name_lst)):
-                # where we are looking for the data container
-                container_path = "/".join(name_lst[:-i])
-                # where we are looking for data in the container
-                data_path = "/".join(name_lst[-1:])
-                yield container_path, data_path
-
-        for container_path, data_path in successive_path_splits(name_lst):
-            try:
-                group = self._hdf5[container_path]
-                if (
-                    isinstance(group, ProjectHDFio)
-                    and "NAME" in group
-                    and group["NAME"] == "DataContainer"
-                ):
-                    return group.to_object(lazy=True)[data_path]
-            except (ValueError, IndexError, KeyError):
-                # either group does not contain a data container or it is does, but it does not have the path we're
-                # looking for
-                pass
-
         item_obj = name_lst[0]
         if item_obj in self._list_ext_childs():
             # ToDo: Murn['strain_0.9'] - sucht im HDF5 file, dort gibt es aber die entsprechenden Gruppen noch nicht.
@@ -962,7 +1098,7 @@ class JobCore(HasGroups):
                 return child["/".join(name_lst[1:])]
         return None
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str, value: Any) -> None:
         """
         Stores data
 
@@ -976,7 +1112,7 @@ class JobCore(HasGroups):
             )
         self._hdf5[key] = value
 
-    def __delitem__(self, key):
+    def __delitem__(self, key: str) -> None:
         """
         Delete item from the HDF5 file
 
@@ -985,7 +1121,7 @@ class JobCore(HasGroups):
         """
         del self.project_hdf5[posixpath.join(self.project_hdf5.h5_path, key)]
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """
         Human readable string representation
 
@@ -994,13 +1130,13 @@ class JobCore(HasGroups):
         """
         return str(self.list_all())
 
-    def _create_working_directory(self):
+    def _create_working_directory(self) -> None:
         """
         internal function to create the working directory on the file system if it does not exist already.
         """
         self.project_hdf5.create_working_directory()
 
-    def _list_ext_childs(self):
+    def _list_ext_childs(self) -> list:
         """
         internal function to list nodes excluding childs
 
@@ -1011,22 +1147,38 @@ class JobCore(HasGroups):
         childs = self.list_childs()
         return list(set(childs) - set(nodes))
 
-    def compress(self, files_to_compress=None):
+    def compress(
+        self,
+        files_to_compress: Optional[List[str]] = None,
+        files_to_remove: Optional[List[str]] = None,
+    ) -> None:
         """
         Compress the output files of a job object.
 
         Args:
             files_to_compress (list):
         """
-        _job_compress(job=self, files_to_compress=files_to_compress)
+        if files_to_compress is None and len(self._files_to_compress) != 0:
+            files_to_compress = self._files_to_compress
+        elif files_to_compress is None:
+            files_to_compress = self.files.list()
+        if files_to_remove is None:
+            files_to_remove = self._files_to_remove
+        else:
+            files_to_remove = []
+        _job_compress(
+            job=self,
+            files_to_compress=files_to_compress,
+            files_to_remove=files_to_remove,
+        )
 
-    def decompress(self):
+    def decompress(self) -> None:
         """
         Decompress the output files of a compressed job object.
         """
         _job_decompress(job=self)
 
-    def is_compressed(self):
+    def is_compressed(self) -> bool:
         """
         Check if the job is already compressed or not.
 
@@ -1035,19 +1187,19 @@ class JobCore(HasGroups):
         """
         return _job_is_compressed(job=self)
 
-    def self_archive(self):
+    def self_archive(self) -> None:
         """
         Compress HDF5 file of the job object to tar-archive
         """
         _job_archive(job=self)
 
-    def self_unarchive(self):
+    def self_unarchive(self) -> None:
         """
         Decompress HDF5 file of the job object from tar-archive
         """
         _job_unarchive(job=self)
 
-    def is_self_archived(self):
+    def is_self_archived(self) -> bool:
         """
         Check if the HDF5 file of the Job is compressed as tar-archive
 
@@ -1055,50 +1207,3 @@ class JobCore(HasGroups):
             bool: [True/False]
         """
         return _job_is_archived(job=self)
-
-
-class DatabaseProperties(object):
-    """
-    Access the database entry of the job
-    """
-
-    def __init__(self, job_dict=None):
-        self._job_dict = job_dict
-
-    def __bool__(self):
-        return self._job_dict is not None
-
-    def __dir__(self):
-        return list(self._job_dict.keys())
-
-    def __getattr__(self, name):
-        if name in self._job_dict.keys():
-            return self._job_dict[name]
-        else:
-            raise AttributeError(name)
-
-    def __repr__(self):
-        return f"{self.__class__.__name__}({repr(self._job_dict)})"
-
-
-class HDF5Content(object):
-    """
-    Access the HDF5 file of the job
-    """
-
-    def __init__(self, project_hdf5):
-        self._project_hdf5 = project_hdf5
-
-    def __getattr__(self, name):
-        if name in self._project_hdf5.list_nodes():
-            return self._project_hdf5.__getitem__(name)
-        elif name in self._project_hdf5.list_groups():
-            return HDF5Content(self._project_hdf5.__getitem__(name))
-        else:
-            raise AttributeError
-
-    def __dir__(self):
-        return self._project_hdf5.list_nodes() + self._project_hdf5.list_groups()
-
-    def __repr__(self):
-        return self._project_hdf5.__repr__()
