@@ -3,7 +3,13 @@
 # Distributed under the terms of "New BSD License", see the LICENSE file.
 
 import os
-from pyiron_base.interfaces.object import HasStorage
+from dataclasses import asdict, fields
+from typing import List, Optional, Tuple, Union
+
+from pyiron_dataclasses.v1.job import Executable as ExecutableDataClass
+from pyiron_snippets.resources import ExecutableResolver
+
+from pyiron_base.interfaces.has_dict import HasDict
 from pyiron_base.state import state
 
 """
@@ -22,14 +28,15 @@ __status__ = "production"
 __date__ = "Sep 1, 2017"
 
 
-class Executable(HasStorage):
+class Executable(HasDict):
+    __hdf_version__ = "0.3.0"
+
     def __init__(
         self,
-        path_binary_codes=None,
-        codename=None,
-        module=None,
-        code=None,
-        overwrite_nt_flag=False,
+        path_binary_codes: Optional[List[str]] = None,
+        codename: Optional[str] = None,
+        module: Optional[str] = None,
+        overwrite_nt_flag: bool = False,
     ):
         """
         Handle the path to the executable, as well as the version selection.
@@ -40,57 +47,40 @@ class Executable(HasStorage):
             overwrite_nt_flag (bool):
         """
         super().__init__()
-        self.storage.table_name = "executable"
+        if overwrite_nt_flag:
+            operation_system_nt = False
+        else:
+            operation_system_nt = os.name == "nt"
+
+        self.storage = ExecutableDataClass(
+            version=None,
+            name=codename.lower(),
+            operation_system_nt=operation_system_nt,
+            executable=None,
+            mpi=False,
+            accepted_return_codes=[0],
+        )
 
         if path_binary_codes is None:
             path_binary_codes = state.settings.resource_paths
-        self.storage.version = None
-        if code is not None:  # Backwards compatibility
-            if not isinstance(code.__name__, str):
-                raise TypeError("The codename should be a string.")
-            codename = code.__name__
-            module = code.__module__.split(".")[1]
-        if codename is not None and module is not None:
-            self.storage.name = codename.lower()
-            code_path_lst = [
-                os.path.join(path, module, "bin") for path in path_binary_codes
-            ]
-            backwards_compatible_path_lst = [
-                os.path.join(path, self.storage.name) for path in path_binary_codes
-            ]
-            self.path_bin = [
-                exe_path
-                for exe_path in (code_path_lst + backwards_compatible_path_lst)
-                if os.path.exists(exe_path)
-            ]
-        else:  # Backwards compatibility
-            self.storage.name = codename.lower()
-            self.path_bin = [
-                os.path.join(path, self.storage.name)
-                for path in path_binary_codes
-                if os.path.exists(os.path.join(path, self.storage.name))
-            ]
-        if overwrite_nt_flag:
-            self.storage.operation_system_nt = False
-        else:
-            self.storage.operation_system_nt = os.name == "nt"
+        if module is None:
+            module = self.storage.name
+        self._module = module
+        self.path_bin = path_binary_codes
         self.executable_lst = self._executable_versions_list()
-        self.storage.executable = None
         self._executable_path = None
-        self.storage.mpi = False
         if self.executable_lst:
             self.version = self.default_version
-        self.storage.accepted_return_codes = [0]
 
     @property
-    def accepted_return_codes(self):
+    def accepted_return_codes(self) -> List[int]:
         """
         list of int: accept all of the return codes in this list as the result of a successful run
         """
         return self.storage.accepted_return_codes
 
     @accepted_return_codes.setter
-    def accepted_return_codes(self, value):
+    def accepted_return_codes(self, value: List[int]) -> None:
         if not isinstance(value, list) or any(
             not isinstance(c, int) or c > 255 for c in value
         ):
@@ -98,7 +88,7 @@ class Executable(HasStorage):
         self.storage.accepted_return_codes = value
 
     @property
-    def version(self):
+    def version(self) -> str:
         """
         Version of the Executable
 
@@ -108,7 +98,7 @@ class Executable(HasStorage):
         return self.storage.version
 
     @property
-    def default_version(self):
+    def default_version(self) -> str:
         """
         Default Version of the Available Executables
         i.e. specifically defined
@@ -122,7 +112,7 @@ class Executable(HasStorage):
         return sorted(self.executable_lst.keys())[0]
 
     @version.setter
-    def version(self, new_version):
+    def version(self, new_version: str) -> None:
         """
         Version of the Executable
 
@@ -142,7 +132,7 @@ class Executable(HasStorage):
             )
 
     @property
-    def mpi(self):
+    def mpi(self) -> bool:
         """
         Check if the message processing interface is activated.
 
@@ -154,7 +144,7 @@ class Executable(HasStorage):
         return self.storage.mpi
 
     @mpi.setter
-    def mpi(self, mpi_bool):
+    def mpi(self, mpi_bool: bool) -> None:
         """
         Activate the message processing interface.
 
@@ -169,7 +159,7 @@ class Executable(HasStorage):
             raise ValueError("No executable set!")
 
     @property
-    def available_versions(self):
+    def available_versions(self) -> List[str]:
         """
         List all available exectuables in the path_binary_codes for the specified codename.
 
@@ -178,7 +168,7 @@ class Executable(HasStorage):
         """
         return self.list_executables()
 
-    def list_executables(self):
+    def list_executables(self) -> List[str]:
         """
         List all available exectuables in the path_binary_codes for the specified codename.
 
@@ -188,7 +178,7 @@ class Executable(HasStorage):
         return sorted(list(self.executable_lst.keys()))
 
     @property
-    def executable_path(self):
+    def executable_path(self) -> str:
         """
         Get the executable path
 
@@ -203,7 +193,7 @@ class Executable(HasStorage):
         return self._executable_select()
 
     @executable_path.setter
-    def executable_path(self, new_path):
+    def executable_path(self, new_path: str) -> None:
         """
         Set the executable path
 
@@ -217,13 +207,50 @@ class Executable(HasStorage):
         else:
             self.storage.mpi = False
 
-    def get_input_for_subprocess_call(self, cores, threads):
+    @classmethod
+    def instantiate(cls, obj_dict: dict, version: str = None) -> "Executable":
+        try:
+            codename = obj_dict["name"]
+        except KeyError:
+            codename = obj_dict["executable"]["name"]
+        return cls(codename=codename)
+
+    def _to_dict(self) -> dict:
+        """
+        Convert the object to a dictionary.
+
+        Returns:
+            dict: A dictionary representation of the object.
+        """
+        return asdict(self.storage)
+
+    def _from_dict(self, obj_dict: dict, version: Optional[str] = None) -> None:
+        """
+        Load the object from a dictionary representation.
+
+        Args:
+            obj_dict (dict): A dictionary representation of the object.
+            version (str, optional): The version of the object. Defaults to None.
+        """
+        data_container_keys = tuple(f.name for f in fields(ExecutableDataClass))
+        executable_class_dict = {}
+        # Backwards compatibility; dict state used to be nested one level deeper
+        if "executable" in obj_dict.keys() and isinstance(obj_dict["executable"], dict):
+            obj_dict = obj_dict["executable"]
+        for key in data_container_keys:
+            executable_class_dict[key] = obj_dict.get(key, None)
+        self.storage = ExecutableDataClass(**executable_class_dict)
+
+    def get_input_for_subprocess_call(
+        self, cores: int, threads: int, gpus: Optional[int] = None
+    ) -> Tuple[Union[str, List[str]], bool]:
         """
         Get the input parameters for the subprocess call to execute the job
 
         Args:
             cores (int): number of cores
             threads (int): number of threads
+            gpus (int/None): number of gpus
 
         Returns:
             str/ list, boolean:  executable and shell variables
@@ -231,68 +258,43 @@ class Executable(HasStorage):
         if cores == 1 or not self.mpi:
             executable = self.__str__()
             shell = True
-        elif isinstance(self.executable_path, list):
-            executable = self.executable_path[:] + [
-                str(cores),
-                str(threads),
-            ]
-            shell = False
         else:
-            executable = [
-                self.executable_path,
-                str(cores),
-                str(threads),
-            ]
+            if isinstance(self.executable_path, list):
+                executable = self.executable_path[:]
+            else:
+                executable = [self.executable_path]
+            executable += [str(cores), str(threads)]
+            if gpus is not None:
+                executable += [str(gpus)]
             shell = False
         return executable, shell
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """
         Executable path
         """
         return repr(self.executable_path)
 
-    def __str__(self):
+    def __str__(self) -> str:
         """
         Executable path
         """
         return str(self.executable_path)
 
-    def _executable_versions_list(self):
+    def _executable_versions_list(self) -> dict:
         """
         Internal function to list all available exectuables in the path_binary_codes for the specified codename.
 
         Returns:
             dict: list of the available version
         """
-        if self.storage.operation_system_nt:
-            extension = ".bat"
-        else:
-            extension = ".sh"
-        try:
-            executable_dict = {}
-            for path in self.path_bin:
-                for executable in os.listdir(path):
-                    if (
-                        executable.startswith("run_" + self.storage.name + "_")
-                        & executable.endswith(extension)
-                        and executable[
-                            len("run_" + self.storage.name) + 1 : -len(extension)
-                        ]
-                        not in executable_dict.keys()
-                    ):
-                        executable_dict[
-                            executable[
-                                len("run_" + self.storage.name) + 1 : -len(extension)
-                            ]
-                        ] = os.path.join(path, executable).replace("\\", "/")
-            return executable_dict
-        except (
-            OSError
-        ):  # No executable exists - This is the case for GenericJob and other abstract job classes.
-            return dict()
+        return ExecutableResolver(
+            resource_paths=self.path_bin,
+            code=self.storage.name,
+            module=self._module,
+        ).dict()
 
-    def _executable_select(self):
+    def _executable_select(self) -> str:
         """
         Internal function to select an executable based on the codename and the version.
 
@@ -302,7 +304,10 @@ class Executable(HasStorage):
         try:
             return self.executable_lst[self.version]
         except KeyError:
-            return ""
+            if isinstance(self.version, str):
+                return self.version
+            else:
+                return ""
 
-    def _get_hdf_group_name(self):
+    def _get_hdf_group_name(self) -> str:
         return "executable"
