@@ -4,13 +4,19 @@
 """
 Helper functions for the JobCore and GenericJob objects
 """
+
 import os
 import posixpath
-import psutil
-import tarfile
-import stat
 import shutil
-from typing import Optional, Union
+import stat
+import tarfile
+from itertools import islice
+from typing import Optional, Tuple, Union
+
+import monty.io
+import psutil
+
+from pyiron_base.database.sqlcolumnlength import JOB_STR_LENGTH
 from pyiron_base.utils.instance import static_isinstance
 from pyiron_base.utils.safetar import safe_extract
 
@@ -26,7 +32,18 @@ __status__ = "production"
 __date__ = "Nov 28, 2020"
 
 
-def _copy_database_entry(new_job_core, job_copied_id):
+_special_symbol_replacements = {
+    ".": "d",
+    "-": "m",
+    "+": "p",
+    ",": "c",
+    " ": "_",
+}
+
+
+def _copy_database_entry(
+    new_job_core: "pyiron_base.jobs.job.generic.GenericJob", job_copied_id: int
+) -> None:
     """
     Copy database entry from previous job
 
@@ -45,7 +62,11 @@ def _copy_database_entry(new_job_core, job_copied_id):
         new_job_core.reset_job_id(job_id=job_id)
 
 
-def _copy_to_delete_existing(project_class, job_name, delete_job):
+def _copy_to_delete_existing(
+    project_class: "pyiron_base.project.generic.Project",
+    job_name: str,
+    delete_job: bool,
+):
     """
     Check if the job exists already in the project, if that is the case either
     delete it or reload it depending on the setting of delete_job
@@ -69,7 +90,19 @@ def _copy_to_delete_existing(project_class, job_name, delete_job):
             return None
 
 
-def _get_project_for_copy(job, project, new_job_name):
+def _get_project_for_copy(
+    job: "pyiron_base.jobs.job.core.JobCore",
+    project: Optional[
+        Union[
+            "pyiron_base.project.generic.Project",
+            "pyiron_base.jobs.job.core.JobCore",
+            "pyiron_base.storage.hdfio.ProjectHDFio",
+        ]
+    ],
+    new_job_name: str,
+) -> Tuple[
+    "pyiron_base.project.generic.Project", "pyiron_base.storage.hdfio.ProjectHDFio"
+]:
     """
     Internal helper function to generate a project and hdf5 project for copying
 
@@ -89,12 +122,12 @@ def _get_project_for_copy(job, project, new_job_name):
     ):
         file_project = project.project
         hdf5_project = project.project_hdf5.open(new_job_name)
-    elif isinstance(project, job.project.__class__):
+    elif isinstance(job.project, project.__class__):
         file_project = project
         hdf5_project = job.project_hdf5.__class__(
             project=project, file_name=new_job_name, h5_path="/" + new_job_name
         )
-    elif isinstance(project, job.project_hdf5.__class__):
+    elif isinstance(job.project_hdf5, project.__class__):
         file_project = project.project
         hdf5_project = project.open(new_job_name)
     elif project is None:
@@ -107,20 +140,11 @@ def _get_project_for_copy(job, project, new_job_name):
     return file_project, hdf5_project
 
 
-_special_symbol_replacements = {
-    ".": "d",
-    "-": "m",
-    "+": "p",
-    ",": "c",
-    " ": "_",
-}
-
-
 def _get_safe_job_name(
     name: Union[str, tuple],
     ndigits: Optional[int] = 8,
     special_symbols: Optional[dict] = None,
-):
+) -> str:
     """
     Sanitize a job name, optionally appending numeric values.
 
@@ -161,7 +185,12 @@ _get_safe_job_name.__doc__ = _get_safe_job_name.__doc__.replace(
 )
 
 
-def _rename_job(job, new_job_name):
+def _rename_job(
+    job: Union[
+        "pyiron_base.jobs.job.generic.GenericJob", "pyiron_base.jobs.job.core.JobCore"
+    ],
+    new_job_name: str,
+) -> None:
     """ """
     new_job_name = _get_safe_job_name(new_job_name)
     child_ids = job.child_ids
@@ -195,7 +224,7 @@ def _rename_job(job, new_job_name):
         )
 
 
-def _is_valid_job_name(job_name):
+def _is_valid_job_name(job_name: str) -> None:
     """
     internal function to validate the job_name - only available in Python 3.4 <
 
@@ -206,14 +235,26 @@ def _is_valid_job_name(job_name):
         raise ValueError(
             f'Invalid name for a pyiron object, must be letters, digits (not as first character) and "_" only, not {job_name}'
         )
-    if len(job_name) > 50:
+    if len(job_name) > JOB_STR_LENGTH:
         raise ValueError(
-            "Invalid name for a PyIron object: must be less then or "
-            "equal to 50 characters"
+            f"Invalid name for a PyIron object: must be less then or equal to {JOB_STR_LENGTH} characters"
         )
 
 
-def _copy_restart_files(job):
+def _get_restart_copy_dict(job: "pyiron_base.jobs.job.generic.GenericJob") -> dict:
+    copy_dict = {}
+    for i, actual_name in enumerate(
+        [os.path.basename(f) for f in job.restart_file_list]
+    ):
+        if actual_name in job.restart_file_dict.keys():
+            new_name = job.restart_file_dict[actual_name]
+        else:
+            new_name = os.path.basename(job.restart_file_list[i])
+        copy_dict[new_name] = job.restart_file_list[i]
+    return copy_dict
+
+
+def _copy_restart_files(job: "pyiron_base.jobs.job.generic.GenericJob") -> None:
     """
     Internal helper function to copy the files required for the restart job.
     """
@@ -221,20 +262,14 @@ def _copy_restart_files(job):
         raise ValueError(
             "The working directory is not yet available to copy restart files"
         )
-    for i, actual_name in enumerate(
-        [os.path.basename(f) for f in job.restart_file_list]
-    ):
-        if actual_name in job.restart_file_dict.keys():
-            new_name = job.restart_file_dict[actual_name]
-            shutil.copy(
-                job.restart_file_list[i],
-                posixpath.join(job.working_directory, new_name),
-            )
-        else:
-            shutil.copy(job.restart_file_list[i], job.working_directory)
+    for file_name, source in _get_restart_copy_dict(job=job).items():
+        shutil.copy(
+            source,
+            posixpath.join(job.working_directory, file_name),
+        )
 
 
-def _kill_child(job):
+def _kill_child(job: "pyiron_base.jobs.job.core.JobCore") -> None:
     """
     Internal helper function to kill a child process.
 
@@ -261,49 +296,70 @@ def _kill_child(job):
                     job_process.kill()
 
 
-def _job_compress(job, files_to_compress=None):
+def _job_compressed_name(job: "pyiron_base.jobs.job.core.JobCore") -> str:
+    """Return the canonical file name of a compressed job."""
+    return _get_compressed_job_name(working_directory=job.working_directory)
+
+
+def _get_compressed_job_name(
+    working_directory: "pyiron_base.jobs.job.core.JobCore",
+) -> str:
+    """Return the canonical file name of a compressed job from the working directory."""
+    return os.path.join(
+        working_directory, os.path.basename(working_directory) + ".tar.bz2"
+    )
+
+
+def _job_compress(
+    job: "pyiron_base.jobs.job.core.JobCore",
+    files_to_compress: list = [],
+    files_to_remove: list = [],
+) -> None:
     """
     Compress the output files of a job object.
 
     Args:
         job (JobCore): job object to compress
         files_to_compress (list): list of files to compress
+        files_to_remove (list): list of files to remove
     """
+
+    def delete_file_or_folder(fullname):
+        if os.path.isfile(fullname):
+            os.remove(fullname)
+        elif os.path.isdir(fullname):
+            shutil.rmtree(fullname)
+
     if not _job_is_compressed(job):
-        if files_to_compress is None:
-            files_to_compress = list(job.list_files())
+        for name in files_to_remove:
+            delete_file_or_folder(fullname=os.path.join(job.working_directory, name))
         cwd = os.getcwd()
         try:
             os.chdir(job.working_directory)
-            with tarfile.open(
-                os.path.join(job.working_directory, job.job_name + ".tar.bz2"),
-                "w:bz2",
-            ) as tar:
+            with tarfile.open(_job_compressed_name(job), "w:bz2") as tar:
                 for name in files_to_compress:
                     if "tar" not in name and not stat.S_ISFIFO(os.stat(name).st_mode):
                         tar.add(name)
             for name in files_to_compress:
-                if "tar" not in name:
-                    fullname = os.path.join(job.working_directory, name)
-                    if os.path.isfile(fullname):
-                        os.remove(fullname)
-                    elif os.path.isdir(fullname):
-                        shutil.rmtree(fullname)
+                if name != _job_compressed_name(job):
+                    delete_file_or_folder(
+                        fullname=os.path.join(job.working_directory, name)
+                    )
         finally:
             os.chdir(cwd)
     else:
-        print("The files are already compressed!")
+        job.logger.info("The files are already compressed!")
 
 
-def _job_decompress(job):
+def _job_decompress(job: "pyiron_base.jobs.job.core.JobCore") -> None:
     """
     Decompress the output files of a compressed job object.
 
     Args:
         job (JobCore): job object to decompress
     """
+    tar_file_name = _job_compressed_name(job)
     try:
-        tar_file_name = os.path.join(job.working_directory, job.job_name + ".tar.bz2")
         with tarfile.open(tar_file_name, "r:bz2") as tar:
             safe_extract(tar, job.working_directory)
         os.remove(tar_file_name)
@@ -311,7 +367,23 @@ def _job_decompress(job):
         pass
 
 
-def _job_is_compressed(job):
+def _working_directory_is_compressed(working_directory: str) -> bool:
+    """
+    Check if the working directory of a given job is already compressed or not.
+
+    Args:
+        working_directory (str): working directory of the job object
+
+    Returns:
+        bool: [True/False]
+    """
+    compressed_name = os.path.basename(
+        _get_compressed_job_name(working_directory=working_directory)
+    )
+    return compressed_name in os.listdir(working_directory)
+
+
+def _job_is_compressed(job: "pyiron_base.jobs.job.core.JobCore") -> bool:
     """
     Check if the job is already compressed or not.
 
@@ -321,14 +393,143 @@ def _job_is_compressed(job):
     Returns:
         bool: [True/False]
     """
-    compressed_name = job.job_name + ".tar.bz2"
-    for name in job.list_files():
-        if compressed_name in name:
-            return True
-    return False
+    return _working_directory_is_compressed(working_directory=job.working_directory)
 
 
-def _job_archive(job):
+def _working_directory_list_files(
+    working_directory: str, include_archive: bool = True
+) -> list:
+    """
+    Returns list of files in the jobs working directory.
+
+    If the working directory is compressed, return a list of files in the archive.
+
+    Args:
+        working_directory (str): working directory of the job object to inspect files in
+        include_archive (bool): include files in the .tag.gz archive
+
+    Returns:
+        list of str: file names
+    """
+    if os.path.isdir(working_directory):
+        uncompressed_files_lst = os.listdir(working_directory)
+        if include_archive and _working_directory_is_compressed(
+            working_directory=working_directory
+        ):
+            compressed_job_name = _get_compressed_job_name(
+                working_directory=working_directory
+            )
+            with tarfile.open(compressed_job_name, "r") as tar:
+                compressed_files_lst = [
+                    member.name for member in tar.getmembers() if member.isfile()
+                ]
+                uncompressed_files_lst.remove(os.path.basename(compressed_job_name))
+                return uncompressed_files_lst + compressed_files_lst
+        else:
+            return uncompressed_files_lst
+    return []
+
+
+def _job_list_files(job: "pyiron_base.jobs.job.core.JobCore") -> list:
+    """
+    Returns list of files in the jobs working directory.
+
+    If the job is compressed, return a list of files in the archive.
+
+    Args:
+        job (JobCore): job object to inspect files in
+
+    Returns:
+        list of str: file names
+    """
+    return _working_directory_list_files(working_directory=job.working_directory)
+
+
+def _working_directory_read_file(
+    working_directory: str, file_name: str, tail: Optional[int] = None
+) -> list:
+    """
+    Return list of lines of the given file.
+
+    Transparently decompresses the file if working directory is compressed.
+
+    If `tail` is given and job is decompressed, only read the last lines
+    instead of traversing the full file.
+
+    Args:
+        working_directory (str): working directory of the job object
+        file_name (str): the file to print
+        tail (int, optional): only return the last lines
+
+    Raises:
+        FileNotFoundError: if the given file name does not exist in the job folder
+    """
+    if file_name not in _working_directory_list_files(
+        working_directory=working_directory
+    ):
+        raise FileNotFoundError(file_name)
+
+    if _working_directory_is_compressed(
+        working_directory=working_directory
+    ) and file_name not in os.listdir(working_directory):
+        with tarfile.open(
+            _get_compressed_job_name(working_directory=working_directory),
+            encoding="utf8",
+        ) as f:
+            lines = [
+                line.decode("utf8") for line in f.extractfile(file_name).readlines()
+            ]
+            if tail is None:
+                return lines
+            else:
+                return lines[-tail:]
+    else:
+        file_name = posixpath.join(working_directory, file_name)
+        if tail is None:
+            with open(file_name) as f:
+                return f.readlines()
+        else:
+            lines = list(
+                reversed(
+                    [
+                        line + os.linesep
+                        for line in islice(monty.io.reverse_readfile(file_name), tail)
+                    ]
+                )
+            )
+            # compatibility with the other methods
+            # monty strips all newlines, where as reading the other ways does
+            # not.  So if a file does not end with a newline (as most text
+            # files) adding it to every line like above adds an additional one.
+            if len(lines) > 0:
+                lines[-1] = lines[-1].rstrip(os.linesep)
+            return lines
+
+
+def _job_read_file(
+    job: "pyiron_base.jobs.job.core.JobCore", file_name: str, tail: Optional[int] = None
+) -> list:
+    """
+    Return list of lines of the given file.
+
+    Transparently decompresses the file if job is compressed.
+
+    If `tail` is given and job is decompressed, only read the last lines
+    instead of traversing the full file.
+
+    Args:
+        file_name (str): the file to print
+        tail (int, optional): only return the last lines
+
+    Raises:
+        FileNotFoundError: if the given file name does not exist in the job folder
+    """
+    return _working_directory_read_file(
+        working_directory=job.working_directory, file_name=file_name, tail=tail
+    )
+
+
+def _job_archive(job: "pyiron_base.jobs.job.core.JobCore") -> None:
     """
     Compress HDF5 file of the job object to tar-archive
 
@@ -357,7 +558,7 @@ def _job_archive(job):
         os.chdir(cwd)
 
 
-def _job_unarchive(job):
+def _job_unarchive(job: "pyiron_base.jobs.job.core.JobCore") -> None:
     """
     Decompress HDF5 file of the job object from tar-archive
 
@@ -374,7 +575,7 @@ def _job_unarchive(job):
         pass
 
 
-def _job_is_archived(job):
+def _job_is_archived(job: "pyiron_base.jobs.job.core.JobCore") -> bool:
     """
     Check if the HDF5 file of the Job is compressed as tar-archive
 
@@ -389,7 +590,7 @@ def _job_is_archived(job):
     )
 
 
-def _job_delete_hdf(job):
+def _job_delete_hdf(job: "pyiron_base.jobs.job.core.JobCore") -> None:
     """
     Delete HDF5 file of job object
 
@@ -400,7 +601,7 @@ def _job_delete_hdf(job):
         os.remove(job.project_hdf5.file_name)
 
 
-def _job_delete_files(job):
+def _job_delete_files(job: "pyiron_base.jobs.job.core.JobCore") -> None:
     """
     Delete files in the working directory of job object
 
@@ -414,7 +615,7 @@ def _job_delete_files(job):
         job._import_directory = None
 
 
-def _job_remove_folder(job):
+def _job_remove_folder(job: "pyiron_base.jobs.job.core.JobCore") -> None:
     """
     Delete the working directory of the job object
 
@@ -426,7 +627,7 @@ def _job_remove_folder(job):
         shutil.rmtree(working_directory)
 
 
-def _job_store_before_copy(job):
+def _job_store_before_copy(job: "pyiron_base.jobs.job.core.JobCore") -> bool:
     """
     Store job in HDF5 file for copying
 
@@ -444,7 +645,9 @@ def _job_store_before_copy(job):
     return delete_file_after_copy
 
 
-def _job_reload_after_copy(job, delete_file_after_copy):
+def _job_reload_after_copy(
+    job: "pyiron_base.jobs.job.core.JobCore", delete_file_after_copy: bool
+) -> None:
     """
     Reload job from HDF5 file after copying
 
