@@ -5,9 +5,8 @@
 The ListMaster behaves like a list, just for job objects.
 """
 
-from pyiron_base.jobs.job.core import JobCore
+from pyiron_base.jobs.job.core import JobCore, _doc_str_job_core_args
 from pyiron_base.jobs.job.generic import GenericJob
-from pyiron_base.jobs.job.core import _doc_str_job_core_args
 from pyiron_base.jobs.master.generic import GenericMaster, _doc_str_generic_master_attr
 from pyiron_base.jobs.master.submissionstatus import SubmissionStatus
 
@@ -219,10 +218,10 @@ class ListMaster(GenericMaster):
         Internal helper function the run if refresh function is called when the job status is 'refresh'. If the job was
         suspended previously, the job is going to be started again, to be continued.
         """
-        self._logger.info(
-            "{}, status: {}, finished: {} parallel master "
-            "refresh".format(self.job_info_str, self.status, self.is_finished())
+        log_str = "{}, status: {}, finished: {} parallel master " "refresh".format(
+            self.job_info_str, self.status, self.is_finished()
         )
+        self._logger.info(log_str)
         if self.is_finished() and not self.server.run_mode.modal:
             self.status.finished = True
         elif (
